#!/bin/env python3

import logging
import time
from usdk import USDK, AREA_CN, AuthInfo, PayResult

logging.basicConfig(format='%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s',
                    level=logging.DEBUG)

PRODUCT_ID = "获取方式请参考接入文档" # 游戏在SDK平台的应用码
PRODUCT_KEY = "获取方式请参考接入文档" # 游戏的APP_KEY
CALLBACK_KEY = "获取方式请参考接入文档" # SDK服务端为游戏的同步的秘钥

# SDK实例，这里默认使用 「国外（global）」
usdk = USDK(PRODUCT_ID, PRODUCT_KEY, CALLBACK_KEY, _logger=logging.info)


def game_srv():
    """
    演示一个简单的游戏服务业务（只有对接SDK功能）
    """
    import flask
    app = flask.Flask(__name__)

    GAME_USER = {}

    def game_on_pay(p: PayResult):
        """
        处理 『充值到账』，这里只是给用户累计道具数
        :param p: 充值结果
        """
        gameuser = '{}_{}'.format(p.channelId, p.channelUid)
        propId = p.goodsId
        if gameuser not in GAME_USER:
            GAME_USER[gameuser] = {}
        if 'pack' not in GAME_USER[gameuser]:
            GAME_USER[gameuser]['pack'] = {}
        GAME_USER[gameuser]['pack'][propId] = GAME_USER[gameuser]['pack'].get(propId, 0) + 1

    @app.route('/login', methods=["POST"])
    def login_check():
        """
        登录校验
        :return:
        """
        cUid = flask.request.values.get('cUid')
        cName = flask.request.values.get('cName')
        accessToken = flask.request.values.get('accessToken')
        result = usdk.login(cUid, cName, accessToken)
        logging.info('USDK校验结果 :{}'.format(result))
        auth_info = AuthInfo.parse(result)
        if auth_info is not None and auth_info.code == 0 and auth_info.cUid:
            logging.info('登录渠道：{}({})'.format(auth_info.channel, auth_info.channelId))
            logging.info('登录渠道用户信息：cUid: {}\tcName: {}'.format(auth_info.cUid, auth_info.cName))

            # 示例：这里定义「游戏」的账号
            gameUser = '{}_{}'.format(auth_info.channelId, auth_info.cUid)
            if not gameUser in GAME_USER:
                GAME_USER[gameUser] = {}
            if not 'sdkuser' in GAME_USER[gameUser]:
                GAME_USER[gameUser]['sdkuser'] = {'channelId': auth_info.channelId,
                                                  'channelUserId': auth_info.cUid,
                                                  'channelUserName': auth_info.cName}

            # 返回登录结果
            return {'code': 0, 'uid': gameUser, 'channel': auth_info.channelId}
        # 这里是登录失败
        return {'code': -1}

    @app.route('/order', methods=["POST"])
    def order():
        """
        游戏 下单
        :return: 游戏订单号
        """
        uid = flask.request.values.get('uid')
        goodsId = flask.request.values.get('goodsId')
        if uid and goodsId:
            sample_order = '{}_{}_{}'.format(uid, goodsId, int(time.time()))
            return {'code': 0, 'order': sample_order}
        return {'code': -1}

    @app.route('/payC', methods=["POST"])
    def pay_callback():
        """
        充值回调，数据来自SDK服务器，以 "application/x-www-form-urlencoded" 形式通知过来
        :return:
        """
        data = flask.request.values.get('data')
        sign = flask.request.values.get('sign')
        suc, resp, ext = usdk.pay_callback(data, sign)
        # TODO: 更新用户的属性
        logging.info("支付结果 :{}".format(resp))
        if suc == 'SUCCESS':
            pr = PayResult.parse(resp)
            game_on_pay(pr)
        return suc

    @app.route("/game_prop", methods=['GET', 'POST'])
    def game_prop():
        """
        演示： 获取 道具ID
        :return:
        """
        return {'code': 0, 'props': [
            {'key': '1000', 'name': '苹果', 'amount': 0.01},
            {'key': '10000', 'name': '香蕉', 'amount': 1.0},
            {'key': 'a03', 'name': '土豆', 'amount': 3.0},
            {'key': 'a04', 'name': '草莓', 'amount': 4.0},
        ], 'msg': '获取成功(纯演示效果，模拟获取游戏内的档位)'}

    @app.route("/game_user", methods=['GET', 'POST'])
    def game_user():
        '''
        演示：返回角色数据
        :return: {code, info: {pack: {propId:count} }}
        '''
        gameuser = flask.request.values.get('uid')
        if gameuser in GAME_USER:
            logging.info("客户端获取用户 {} : {}".format(gameuser, GAME_USER[gameuser]))
            return {'code': 0, 'info': GAME_USER[gameuser]}
        return {'code': 1, 'msg': 'miss user'}

    @app.route("/game_user_free", methods=['GET', 'POST'])
    def game_user_free():
        '''
        演示：玩家销号
        :return: {code, msg}
        '''
        gameuser = flask.request.values.get('uid')
        if gameuser in GAME_USER:
            logging.info("客户端要求销号 {}".format(gameuser))
            del GAME_USER[gameuser]
            return {'code': 0, 'msg': 'ok'}
        return {'code': 1, 'msg': 'miss user'}

    app.run(host='0.0.0.0')

if __name__ == '__main__':
    game_srv()
